local item_cnt = reaper.CountSelectedMediaItems( 0 )
if item_cnt == 0 then return end

local items, it = {}, 0
local track = reaper.GetMediaItemTrack( reaper.GetSelectedMediaItem( 0, 0 ))

for i = 0, item_cnt-1 do
  local item = reaper.GetSelectedMediaItem( 0, i )
  if reaper.GetMediaItemTrack( item ) ~= track then
    reaper.MB("All selected items must be on the same track", "Aborting...", 0)
    return
  end
  local take = reaper.GetActiveTake( item )
  if take and not reaper.TakeIsMIDI( take ) then
    it = it + 1
    items[it] = {item, take}
  end
end

if it == 0 then return end


for i = 1, it do

  local item = items[i][1]
  local take = items[i][2]

  -- sibilants 14dB below rms will be discarded
  local threshold = 10^((reaper.NF_GetMediaItemAverageRMS( item ) - 14)/20)
  
  local item_pos = reaper.GetMediaItemInfo_Value( item, "D_POSITION" )
  local item_len = reaper.GetMediaItemInfo_Value( item, "D_LENGTH" )
  local item_end = item_pos + item_len
  local source = reaper.GetMediaItemTake_Source( take )
  local nch = reaper.GetMediaSourceNumChannels( source )
  local samplerate = reaper.GetMediaSourceSampleRate( source )
  local ns = 1024
  local buf = reaper.new_array( ns * 3 * nch )
  local accessor = reaper.CreateTakeAudioAccessor( take )
  local acc_buf_sz = ns * nch
  local acc_buf = reaper.new_array( acc_buf_sz )
  -- window length will be half of the ns block (so that windows overlap 50%)
  local window = (ns / samplerate) / 2
  
  -- Calculate cuts
  local cuts, c = {}, 0
  local pos = item_pos
  while pos < item_end do
    local rv = reaper.GetMediaItemTake_Peaks( take, 1000, pos, nch, ns, 115, buf )
    if rv & ( 1 << 24 ) and ( rv & 0xfffff ) > 0 then
      local spl = buf[nch*ns*2 + 1]
      local pitch = spl & 0x7fff
      local tonality = ( spl >> 15 ) / 16384
      -- conditions to consider a split:
      if pitch > 1850 or tonality < 0.15 then
        -- RMS must be above threshold
        local ok = reaper.GetAudioAccessorSamples( accessor, samplerate, nch, pos - item_pos , ns, acc_buf )
        if ok == 1 then
          local rms = 0
          for i = 1, acc_buf_sz do
            rms = rms + acc_buf[i]*acc_buf[i]
          end
          rms = (rms/acc_buf_sz)^(1/2)
          if rms >= threshold then
            if not cut_start then cut_start = pos - 0.01 end -- (0.01: cut a bit earlier)
          end
        end
      else
        -- if the resulting item is 50+ ms, register the splits
        if cut_start and pos - cut_start >= 0.05 then
          c = c + 2
          cuts[c-1] = cut_start
          cuts[c] = pos
        end
        cut_start = nil
      end
    end
    pos = pos + window
  end
  
  reaper.DestroyAudioAccessor( accessor )
  -- Store info to table
  items[i][2] = c
  items[i][3] = cuts
  
end


-- Split items
reaper.Undo_BeginBlock()
reaper.PreventUIRefresh( 1 )

-- Enable autocrossfade
local auto_xfade = reaper.GetToggleCommandState( 40912 ) == 1
if not auto_xfade then
  reaper.Main_OnCommand(40927, 0) -- Enable auto-crossfade on split
end

reaper.SelectAllMediaItems( 0, false )
local split_items, s = {}, 0


for i = 1, it do

  local c = items[i][2]
  if c ~= 0 then
  
    local cuts = items[i][3]
    local item = items[i][1]
    local splt = 0 -- split counter, makes a new item every two splits
    
    local next_item = item -- initialize item to split
    local i = 1 -- index for cuts table
    
    while i <= c do
     
     local even = splt % 2 == 0
     
      -- after having done the first split:
      -- if the next split item is too close ( less than 50ms ) then skip the split (merge the splits)
      if even == false and cuts[i+1] and cuts[i+1] - cuts[i] <= 0.05 then
        i = i + 2
        -- continue merging splits if needed
        while true do
          if cuts[i+1] and cuts[i+1] - cuts[i] <= 0.05 then
            i = i + 2
          else
            break
          end
        end
      end
    
      next_item = reaper.SplitMediaItem( next_item, cuts[i] )
      
      -- every two splits (even splt) store the resulting sibilance item
      if even then
        s = s + 1
        split_items[s] = next_item
      end
      
      splt = splt + 1
      i = i + 1
    
    end
  end

end

if s ~= 0 then
  -- Create child track
  local track_id = reaper.CSurf_TrackToID( track, false )
  local folder_depth = reaper.GetMediaTrackInfo_Value( track, "I_FOLDERDEPTH" )
  reaper.SetMediaTrackInfo_Value( track, "I_FOLDERDEPTH", folder_depth + 1 )
  reaper.InsertTrackAtIndex( track_id, true )
  local new_track = reaper.CSurf_TrackFromID( track_id + 1, false )
  reaper.GetSetMediaTrackInfo_String( new_track, "P_NAME", "sibilance", true )
  reaper.SetMediaTrackInfo_Value( new_track, "I_FOLDERDEPTH", folder_depth - 1 )
  
  
  -- Move sibilance items to child track
  for i = 1, s do
    reaper.MoveMediaItemToTrack( split_items[i], new_track )
  end
end

-- Restore autoxfade
if not auto_xfade then
  reaper.Main_OnCommand(40928, 0) -- Disable auto-crossfade on split
end

reaper.PreventUIRefresh( -1 )
reaper.UpdateArrange()
reaper.Undo_EndBlock( "Split & move sibilance to child track", 4 )